<?php

namespace SuperRhino\AppStoreServerApi;

use Firebase\JWT\JWT;
use Firebase\JWT\Key;

class Util {

    public static function decodeJWS(string $jws, string $applePemString) {
        $components = explode('.', $jws);
        $header = base64_decode($components[0]);

        $headerJson = json_decode($header, true);
        $algorithm = $headerJson['alg'];
        $x5cArray = $headerJson['x5c'];

        foreach ($x5cArray as $X5C) {
            $certificate = '-----BEGIN CERTIFICATE-----' . PHP_EOL;
            $certificate .= chunk_split($X5C, 64, PHP_EOL);
            $certificate .= '-----END CERTIFICATE-----' . PHP_EOL;
            $certificates[] = openssl_x509_read($certificate);//OpenSSLCertificate
        }
        $certificates[] = openssl_x509_read($applePemString);
        //$certificates = array_reverse($certificates);

        // 验证证书链
        foreach ($certificates as $key => $certificate) {
            if ($key < count($certificates) - 1) {
                if (openssl_x509_verify($certificate, $certificates[$key + 1]) != 1) {
                    throw new SignatureInvalidException('Certificate verification failed');
                }
            }
        }

        //第一个证书是签署jws的证书
        $pkey_object = openssl_pkey_get_public($certificates[0]);
        $pkey_array = openssl_pkey_get_details($pkey_object);
        $publicKey = $pkey_array['key'];
        //传入jws以及公钥匙以及加密算法
        $decoded = JWT::decode($jws, new Key($publicKey, $algorithm));

        return json_decode(json_encode($decoded), true);
    }

    public static function unsafeDecodeJWS($jws) {
        $tks = explode('.', $jws);
        if (count($tks) != 3) {
            throw new \UnexpectedValueException('Wrong number of segments');
        }

        $payloadRaw = static::urlsafeB64Decode($tks[1]);
        if (null === ($payload = json_decode($payloadRaw, false, 512, JSON_BIGINT_AS_STRING))) {
            throw new \UnexpectedValueException('Invalid claims encoding');
        }
        return json_decode(json_encode($payload), true);
    }

    /**
     * Decode a string with URL-safe Base64.
     *
     * @param string $input A Base64 encoded string
     *
     * @return string A decoded string
     *
     * @throws \InvalidArgumentException invalid base64 characters
     */
    private static function urlsafeB64Decode(string $input): string {
        $remainder = \strlen($input) % 4;
        if ($remainder) {
            $padlen = 4 - $remainder;
            $input .= \str_repeat('=', $padlen);
        }
        return \base64_decode(\strtr($input, '-_', '+/'));
    }

    /**
     * Encode a string with URL-safe Base64.
     *
     * @param string $input The string you want encoded
     *
     * @return string The base64 encode of what you passed in
     */
    private static function urlsafeB64Encode(string $input): string {
        return \str_replace('=', '', \strtr(\base64_encode($input), '+/', '-_'));
    }
}